package com.hungraim.ltc.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.assessment.ChAprTask;
import com.hungraim.ltc.pojo.entity.disable.*;
import com.hungraim.ltc.pojo.entity.serviceManage.ChCfgPara;
import com.hungraim.ltc.pojo.vo.disable.*;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.service.DisableService;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Slf4j
@AllArgsConstructor
public class DisableServiceImpl implements DisableService {

    private final DisableApplyMapper disableApplyMapper;
    private final FndDistrictMapper districtMapper;
    private final FndOrganMapper fndOrganMapper;
    private final FndStreetMapper fndStreetMapper;
    private final SrvModeMapper srvModeMapper;
    private final SrvOrganMapper srvOrganMapper;
    private final FndDiseaseTypeMapper fndDiseaseTypeMapper;
    private final DisableApplyHisMapper disableApplyHisMapper;
    private final DisableInfoMapper disableInfoMapper;
    private final AprTaskMapper aprTaskMapper;
    private final CfgParaMapper cfgParaMapper;
    private final AttachFeignService attachFeignService;

    /**
     * 获取机构ids
     *
     * @param organId  父机构id
     * @param organIds 机构list
     */
    private void getOrganList(String organId, List<String> organIds) {
        QueryWrapper<ChFndOrgan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("STATUS", "1");
        queryWrapper.eq("PARENT_ID", organId);
        List<ChFndOrgan> organs = fndOrganMapper.selectList(queryWrapper);
        if (organs != null && organs.size() != 0) {
            for (ChFndOrgan chFndOrgan : organs) {
                String childOrganId = chFndOrgan.getOrganId();
                organIds.add(childOrganId);
                if ("1".equals(chFndOrgan.getIsLeaf())) {
                    getOrganList(childOrganId, organIds);
                }
            }
        }
    }

    /**
     * 查询所有资源对应的角色
     *
     * @param organId            适用机构代码
     * @param districtProvincial 所在省
     * @param districtCity       所在市
     * @param districtArea       所在区域
     * @param realName           失能人员姓名
     * @param certiCode          失能人员身份证号码
     * @param dataFrom           申请来源
     * @param srvOrganName       护理机构名称
     * @return list
     */
    @Override
    public Page<DisableVO> disableList(Page<DisableVO> page, String organId, String districtProvincial, String districtCity,
                                       String districtArea, String realName, String certiCode, String dataFrom,
                                       String srvOrganName) {
        // 没有organId参数的时候，这个集合就为null
        List<String> organIds = null;
        if (StrUtil.isNotBlank(organId)) {
            organIds = Arrays.asList(organId.split(","));
        }
        return disableApplyMapper.disableList(page, organIds, districtProvincial,
                districtCity, districtArea, realName, certiCode, dataFrom, srvOrganName);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public DisableExamineInfoVO disableExamineInfo(String applyId) {
        DisableExamineInfoVO disableExamineInfoVO = disableApplyMapper.selectDisableExamineInfoByApplyId(applyId);
        List<ExamineRecord> examineRecords = disableApplyMapper.listExamineRecordByApplyId(applyId);
        disableExamineInfoVO.setExamineRecords(examineRecords);
        try {
            // zhangch 20210525 新增返回对应的附件列表
            Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(Long.valueOf(applyId), AttachType.DISABLE.getUploadType());
            if (listResult.getCode().equals(ResultCode.SUCCESS.getCode())) {
                disableExamineInfoVO.setFileDataList(listResult.getData());
            } else {
                log.info("请求获取附件列表错误：错误码{},错误信息:{}", listResult.getCode(), listResult.getMsg());
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
        }
        return disableExamineInfoVO;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addDisable(ChDisableApply chDisableApply) {
        chDisableApply.setEffTime(new Date());
        disableApplyMapper.insert(chDisableApply);
        this.updateAttachs(chDisableApply);

    }

    /**
     * 调用feign api来修改附件数据
     *
     * @param chDisableApply
     */
    private void updateAttachs(ChDisableApply chDisableApply) {
        // zhangch 20210525 新增附件修改逻辑
        List<ChFndAttach> fileDataList = chDisableApply.getFileDataList();
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setCtrlId(chDisableApply.getApplyId());
        attachUpdateVo.setAttachType(AttachType.DISABLE.getUploadType());
        attachUpdateVo.setChFndAttachs(fileDataList);
        attachFeignService.updateAttachs(attachUpdateVo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void disableExamine(String applyId, String checkName, Short applyStatus, String checkRemark) {
        Date date = new Date();
        //更新审核状态
        ChDisableApply chDisableApply = disableApplyMapper.selectById(applyId);
        chDisableApply.setCheckName(checkName);//审核人
        chDisableApply.setApplyStatus(applyStatus);//申请状态
        chDisableApply.setCheckRemark(checkRemark);//审核备注
        chDisableApply.setCheckTime(date);//审核时间
        disableApplyMapper.updateById(chDisableApply);
        //添加失能审核历史
        ChDisableApplyHis chDisableApplyHis = new ChDisableApplyHis();
        BeanUtils.copyProperties(chDisableApply, chDisableApplyHis);
        disableApplyHisMapper.insert(chDisableApplyHis);
        //审核通过
        if (applyStatus.equals(Short.valueOf("2"))) {
            //添加失能人员信息
            ChDisableInfo chDisableInfo = new ChDisableInfo();
            BeanUtil.copyProperties(chDisableApply, chDisableInfo);
            disableInfoMapper.insert(chDisableInfo);
            //添加失能评定信息
            ChAprTask chAprTask = new ChAprTask();
            ChCfgPara chCfgPara = cfgParaMapper.selectOne(new QueryWrapper<ChCfgPara>().eq("ORGAN_ID", chDisableApply.getOrganId()));
            if (chCfgPara != null && chCfgPara.getCfgArgTime() != null) {
                Calendar cd = Calendar.getInstance();
                cd.setTime(date);
                cd.add(Calendar.DATE, chCfgPara.getCfgArgTime().intValue());
                chAprTask.setLastAprDate(cd.getTime());//鉴定截至日期
            }
            BeanUtil.copyProperties(chDisableApply, chAprTask);
            chAprTask.setDisableInfoId(chDisableInfo.getDisabInfoId());//失能人员id
            chAprTask.setEffTime(new Date());
            chAprTask.setFcd(null);
            aprTaskMapper.insert(chAprTask);
        }
    }

    @Override
    public List<ChFndDistrict> district(String districtLevel, String districtCode) {
        if ("4".equals(districtLevel)) {
            //查询街道
            QueryWrapper<ChFndStreet> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("IS_VALID", 0);
            queryWrapper.eq("AREA_CODE", districtCode);
            List<ChFndStreet> fndStreets = fndStreetMapper.selectList(queryWrapper);
            List<ChFndDistrict> resuts = new ArrayList<>();
            for (ChFndStreet chFndStreet : fndStreets) {
                ChFndDistrict chFndDistrict = new ChFndDistrict();
                chFndDistrict.setDistrictCode(chFndStreet.getStreetCode());
                chFndDistrict.setDistrictName(chFndStreet.getStreetName());
                chFndDistrict.setParentDistirctCode(chFndStreet.getAreaCode());
                resuts.add(chFndDistrict);
            }
            return resuts;
        } else {
            QueryWrapper<ChFndDistrict> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("IS_VALID", "Y");
            queryWrapper.eq("DISTRICT_LEVEL", districtLevel);
            if ("1".equals(districtLevel)) {
                //查出所有的省
                return districtMapper.selectList(queryWrapper);
            } else {
                //查询市/县
                queryWrapper.eq("PARENT_DISTIRCT_CODE", districtCode);
                return districtMapper.selectList(queryWrapper);
            }
        }
    }

    @Override
    public List<ChFndOrgan> fndOrganRoot() {
        QueryWrapper<ChFndOrgan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_LEAF", "1");
        queryWrapper.eq("STATUS", "1");
        queryWrapper.and(wrapper -> wrapper.isNull("PARENT_ID").or().eq("PARENT_ID", ""));
        return fndOrganMapper.selectList(queryWrapper);
    }

    @Override
    public List<SrvModeVo> cfgSrvModeList() {
        return srvModeMapper.selectAll();
    }

    @Override
    public List<ChSrvOrgan> srvOrganList(String organId) {
        return srvOrganMapper.selectList(new QueryWrapper<ChSrvOrgan>().eq("organ_id", organId).eq("STATUS", (short) 0));
    }

    @Override
    public List<ChFndDiseaseType> fndDiseaseList(String parDiseTypeId) {
        QueryWrapper<ChFndDiseaseType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_VALID", "0");
        if (StringUtils.isNotBlank(parDiseTypeId)) {
            queryWrapper.eq("PAR_DISE_TYPE_ID", parDiseTypeId);
        } else {
            queryWrapper.and(wrapper -> wrapper.isNull("PAR_DISE_TYPE_ID").or().eq("PAR_DISE_TYPE_ID", ""));
        }
        return fndDiseaseTypeMapper.selectList(queryWrapper);
    }

    @Override
    public String subsidiariesOrganId(String organId) {
        List<String> organIds = new ArrayList<>();
        organIds.add(organId);
        // 获取子孙机构集合
        this.getOrganList(organId, organIds);
        StringBuilder sb = new StringBuilder();
        for (String o : organIds) {
            sb.append(o).append(",");
        }
        sb.deleteCharAt(sb.length() - 1);
        return sb.toString();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateDisable(ChDisableApply chDisableApply) {
        disableApplyMapper.updateById(chDisableApply);
        // zhangch 20210525 新增附件修改逻辑
        this.updateAttachs(chDisableApply);
    }

    @Override
    public List<DistrictTreeVo> allDistrict() {
        return districtMapper.getAllDistrictAndStreetData();
    }

    @Override
    public List<DistrictTreeVo> getAllDistrictTreeData() {
        return districtMapper.getAllDistrictTreeData();
    }

    @Override
    public List<ChFndDiseaseType> fndAllDisease() {
        QueryWrapper<ChFndDiseaseType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_VALID", "0");
        return fndDiseaseTypeMapper.selectList(queryWrapper);
    }

    @Override
    public DisableInfoVO getDisableInfo(String disableInfoId) {
        return disableApplyMapper.getDisableInfo(disableInfoId);
    }

    @Override
    public List<ChFndOrgan> fndAllOrgan() {
        QueryWrapper<ChFndOrgan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_LEAF", "1");
        queryWrapper.eq("STATUS", "1");
        return fndOrganMapper.selectList(queryWrapper);
    }
}